/**
 * Copyright (C) 2010 OSGi fuer Praktiker <info@osgi-buch.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package biz.gossipmonger.skilledworker.internal;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

import org.osgi.framework.BundleContext;
import org.osgi.service.component.ComponentContext;

import biz.gossipmonger.enunciator.EnunciatorService;
import biz.gossipmonger.skilledworker.SkilledWorkerService;

public final class SkilledWorkerServiceImpl
    implements SkilledWorkerService
{
	HtmlRenderer renderer;
	EnunciatorService enunciator;
	String outputDir;

	/**
	 * @param bundleContext set by framework
	 * @param outputDir	sets the output directory via JRE vmArgs
	 */
	public SkilledWorkerServiceImpl(BundleContext bundleContext, String outputDir, HtmlRenderer renderer) {
		this.renderer = renderer;
		this.outputDir = outputDir;
	}
	
	// Component methods

	protected void activate(ComponentContext context) {
		enunciate("activated.");
	}

	protected void deactivate(ComponentContext context) {
		enunciate("deactivated.");
	}

	public void setEnunciatorService(EnunciatorService es) {
		if (es != null) {
			enunciator = es;
			enunciate("EnunciatorService set.");
		} else {
			enunciate("setEnunciatorService called with null argument.");
		}
	}

	public void unsetEnunciatorService(EnunciatorService es) {
		enunciate("EnunciatorService unset.");
		enunciator = null;
	}

	// The WorkerService methods

	public File process(File file) {
		return process(file, outputDir);
	}

	public File process(File file, String outputDir) {
		enunciate("Render from "+file.getAbsolutePath());
		try {
			return process(new FileInputStream(file), file.getName(), outputDir);
		} catch (FileNotFoundException e) {
			enunciate(file.getAbsolutePath() + " is not there!");
			return null;
		}
	}

	public File process(InputStream is, String inputFilename) {
		return process(is, inputFilename, outputDir);
	}

	public File process(InputStream is, String inputFilename, String outputDir) {
		enunciate("Rendering...");
		try {
			File output = renderer.render(is, inputFilename, outputDir);
			enunciate("Rendered to "+output.getAbsolutePath());
			return output;
		} catch (Exception ex) {
			enunciate("Error while rendering: "+ex.getMessage());
			return null;
		}
	}

	// Internal methods

	private void enunciate(String text) {
		if (enunciator != null) {
			enunciator.enunciate("SkilledWorkerServiceImpl: " + text);
		} else {
			System.out.println("SkilledWorkerServiceImpl(no Enunciator): " + text);
		}
	}
}

