/**
 * Copyright (C) 2010 OSGi fuer Praktiker <info@osgi-buch.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package biz.gossipmonger.worker.internal;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.pdf.PdfWriter;

public class PdfRenderer {

  public File render(InputStream is, String inputFilename, String outputDir) throws IOException {
    FileOutputStream outputStream = null;
    Document doc = null;
    try {
      String content = readStream(is);
      File outputFile = createOutputFile(inputFilename, outputDir);
      outputStream = new FileOutputStream(outputFile);
      doc = new Document();
      PdfWriter.getInstance(doc, outputStream);
      doc.open();
      doc.addTitle("WorkerService PdfRenderer Result");
      doc.add(new Paragraph(content));
      return outputFile;
    } catch (DocumentException e) {
      throw new IOException(e);
    } finally {
      if (doc != null) {
        doc.close();
      }
      if (outputStream != null) {
        outputStream.close();
      }
    }
  }

  private File createOutputFile(String inputFilename, String outputDir) throws IOException {
    File file = new File(inputFilename);
    String name = file.getName();
    String basename = name.replaceFirst("(.*)\\.[^.]*", "$1.pdf");
    File outputDirFile = new File(outputDir);
    if (!outputDirFile.exists() && !outputDirFile.mkdirs()) {
      throw new IOException("Could not create directory " + outputDirFile.getAbsolutePath());
    }
    return new File(outputDir, basename);
  }

  private String readStream(InputStream is) throws IOException {
    StringBuffer sb = new StringBuffer();
    char[] chars = new char[2048];
    BufferedReader buf = new BufferedReader(new InputStreamReader(is));
    int charsRead = 0;
    while ((charsRead = buf.read(chars)) > 0) {
      sb.append(chars, 0, charsRead);
    }
    return sb.toString();
  }
}
