/**
 * Copyright (C) ${year} OSGi für Praktiker <info@osgi-buch.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package biz.gossipmonger.jobtimer;

import java.lang.management.ManagementFactory;
import java.util.Calendar;
import java.util.Dictionary;
import java.util.Hashtable;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventAdmin;
import org.osgi.util.tracker.ServiceTracker;

public class JobTimer implements BundleActivator {

  private Thread t = null;
  private MBeanServer mbs;
  private ObjectName name;

  public void start(BundleContext context) {
    System.out.println("JobTimer is starting...");
    JobTimerConfig jobtimerconfig = new JobTimerConfig(context);
    try {
      mbs = ManagementFactory.getPlatformMBeanServer();
      name = new ObjectName("biz.gossipmonger.jobtimer:"
                            +"Name=JobTimerConfig");
      mbs.registerMBean(jobtimerconfig, name);
      System.out.println("Registered"
                         +" JobTimerConfigMBean");
    } catch (Exception e) {
      e.printStackTrace();
      System.out.println("Cannot create registered"
                         +" JobTimerConfigMBean");
    }
    Runnable task =
        new JobTimerRunnable(context,jobtimerconfig);
    t = new Thread(task);
    t.start();
    System.out.println("JobTimer has been started.");
  }

  public void stop(BundleContext context) {
    System.out.println("JobTimer is stopping...");
    t.interrupt();
    try {
      t.join();
    } catch (InterruptedException e1) {
      System.out.println("Cannot join JobTimerRunnable");
      e1.printStackTrace();
    }
    try {
      mbs.unregisterMBean(name);
      System.out.println("Unregistered"
                         +" JobTimerConfigMBean");
    } catch (Exception e) {
      e.printStackTrace();
      System.out.println("Cannot unregister"
                         +" JobTimerConfigMBean");
    }
    System.out.println("JobTimer has been stopped.");
  }

  private class JobTimerRunnable implements Runnable {

    private Dictionary<String, Integer> time =
        new Hashtable<String, Integer>();
    private BundleContext context;
    private JobTimerConfig jobtimerconfig;

    public JobTimerRunnable(BundleContext context, JobTimerConfig jobtimerconfig) {
      this.context = context;
      this.jobtimerconfig = jobtimerconfig;
    }

    public void run() {
      ServiceTracker tracker = new ServiceTracker(context, EventAdmin.class.getName(), null);
      tracker.open();
      try {
        while (true) {
          Calendar c = Calendar.getInstance();
          time.put("minutes", c.get(Calendar.MINUTE));
          time.put("hours", c.get(Calendar.HOUR));
          time.put("day", c.get(Calendar.DAY_OF_MONTH));
          time.put("month", c.get(Calendar.MONTH));
          time.put("year", c.get(Calendar.YEAR));

          EventAdmin ea =
               (EventAdmin) tracker.getService();
          if (ea != null) {
            Event ev =
                new Event("biz/gossipmonger/jobtimer", time);
            ea.sendEvent(ev);
            System.out.println(ev.toString());
          } else {
            System.out.println("No EventAdmin");
          }
          Thread.sleep(jobtimerconfig.getInterval());
        }
      } catch (InterruptedException e) {
        System.out.println("JobTimer is interrupted.");
      } finally {
        tracker.close();
      }
    }
  }
}
